#if DEBUG > 0

#include "debug.h"
#include <stdio.h>
#include <unistd.h>
#include <stdarg.h>
#include <ctype.h>

static FILE *debug_log = NULL;

/*****************************************************************************/
/*                                                                           */
/*  DebugInit                                                                */
/*                                                                           */
/*****************************************************************************/
void
DebugInit(FILE *f)
{
  debug_log = f;
}

/*****************************************************************************/
/*                                                                           */
/*  DebugPrintf  */
/*                                                                           */
/*****************************************************************************/
void
DebugPrintf(const char *s, ...)
{
  va_list ap;

  if (!debug_log) {
    fprintf(stderr, "No debug log\n");
    exit(1);
  }
  
  va_start(ap, s);
  vfprintf(debug_log, s, ap);
  fflush(debug_log);
  va_end(ap);

#if DEBUG > 3
  va_start(ap, s);
  vfprintf(stderr, s, ap);
  fflush(stderr);
  va_end(ap);
#endif
}

/*****************************************************************************/
/*                                                                           */
/*  AssertPrintf                                                             */
/*                                                                           */
/*****************************************************************************/
void
AssertPrintf(const char *file, int line, const char *s, ...)
{
  va_list ap;

  DebugPrintf("%s:%d:\n", file, line);

  /* I have to repeat debug_printf here, because I can't pass the rest to it */

  if (!debug_log) {
    fprintf(stderr, "No debug log\n");
    exit(1);
  }

  va_start(ap, s);
  vfprintf(debug_log, s, ap);
  fflush(debug_log);
  va_end(ap);

#if DEBUG > 3
  va_start(ap, s);
  vfprintf(stderr, s, ap);
  fflush(stderr);
  va_end(ap);
#endif
}

/*****************************************************************************/
/*                                                                           */
/*  AssertFailed                                                             */
/*  In a debugger, you should put a breakpoint here.                         */
/*                                                                           */
/*****************************************************************************/
void 
AssertFailed(const char *s, int l)
{
  static Bool do_exit = True;
  DebugPrintf("Hey, this should not happen!\n");
  if (do_exit)
    exit(1);
}


/*****************************************************************************/
/*                                                                           */
/*  AssertStr                                                                */
/*                                                                           */
/*****************************************************************************/
void
_AssertStr(const char *f, int l, const char *s)
{
  if (!s) {
    AssertPrintf(f,l, "String is NULL\n");
    AssertFailed(f,l);
  }
  if (strlen(s) > 0x10000) {
    AssertPrintf(f,l, "String > 64k\n");
    AssertFailed(f,l);
  }
}
    
/*****************************************************************************/
/*                                                                           */
/*  AssertPrintStr                                                           */
/*                                                                           */
/*****************************************************************************/
void     
_AssertPrintStr(const char *f, int l, const char *s)
{
  const char *p;
  
  _AssertStr(f,l,s);

  if (!s) return;

  for (p=s; *p; p++) {
    if (!isprint(*p)) {
      AssertPrintf(f,l, "Char should be printable: %d\n", *p);
      AssertFailed(f,l);
    }
  }
}
/*****************************************************************************/
/*                                                                           */
/*  AssertPtr                                                                */
/*                                                                           */
/*****************************************************************************/
void
_AssertPtr(const char *f, int l, const void *const p)
{
  if (!p) {
    AssertPrintf(f,l, "Pointer is NULL\n");
    AssertFailed(f,l);
  }
}


/*****************************************************************************/
/*                                                                           */
/*  The smblib - specific assertions                                         */
/*                                                                           */
/*****************************************************************************/

/*****************************************************************************/
/*                                                                           */
/*  AssertConnSpace                                                          */
/*                                                                           */
/*****************************************************************************/

void _AssertConnSpace(const char *f, int l,
                      smb_conn *conn,
                      const char *p,
                      int length)
{
  _AssertPtr(f,l,conn);
  _AssertPtr(f,l,p);

  if (   (p >= conn->buffer)
      && (conn->buffer + conn->max_xmit < p + length)) {
    AssertPrintf(f,l, "Not enough space in conn-buffer for:\n%s\n", p);
    AssertFailed(f,l);
  }
}

/*****************************************************************************/
/*                                                                           */
/*  AssertConn                                                               */
/*                                                                           */
/*****************************************************************************/
void
_AssertConn(const char *f, int l, smb_conn *conn)
{
  _AssertPtr(f,l,conn);
  if (!conn) return;

#if DEBUG > 1
  if (   (conn->pid != getpid())
      || (conn->uid != getuid())
      || (conn->gid != getgid())
      || (conn->mid != conn->pid + 20)) {
    AssertPrintf(f,l, "Conn values inconsistent\n");
    AssertFailed(f,l);
  }
}
#endif /* DEBUG > 1 */
  
#endif /* DEBUG > 0 */
